# AUTOGENERATED! DO NOT EDIT! File to edit: 01_blocker.ipynb (unless otherwise specified).

__all__ = ['APP_NAME', 'REDIRECT', 'WIN_PATH', 'LINUX_PATH', 'ICON_PATH', 'BLOCKLIST', 'host_fp', 'host_fp_copy',
           'host_fp_blocked', 'Blocker']

# Cell
from pathlib import Path
from shutil import copy
import sys
try:
    from win10toast import ToastNotifier
    win_notify = ToastNotifier()
except:
    win_notify = False

# Cell
APP_NAME = 'Website Blocker'
REDIRECT = '127.0.0.1'
WIN_PATH = r'C:\Windows\System32\drivers\etc'
LINUX_PATH = r'/etc'
ICON_PATH = 'icon.ico'
BLOCKLIST = 'blocklist.txt'

if sys.platform == 'win32':
    host_path = Path(WIN_PATH)
else:
    host_path = Path(LINUX_PATH)
host_fp = host_path/'hosts'
host_fp_copy = host_path/'hosts.original'
host_fp_blocked = host_path/'hosts.blocked'

# Cell
class Blocker():
    "The core of the package. It modifies the hosts file of the OS."
    def __init__(self, redirect=REDIRECT):
        self.adminrights = False
        self.redirect = redirect
        with open(BLOCKLIST) as file:
            self.blocked = [line.rstrip() for line in file]
        if not host_fp_copy.exists():
            self._setup()
        if self._create_blocked_list():
            self.adminrights = True

    def _setup(self):
        "Creates a copy of the `hosts` file and saves it as `hosts.original`"
        try:
            copy(host_fp, host_fp_copy)
            self.notify("Setup successful")
        except PermissionError:
            self._raise_permission_error()

    def _create_blocked_list(self):
        "Creates a copy of `hosts.original` and saves it to `hosts.blocked`. Then adds all blocked sites."
        try:
            copy(host_fp_copy, host_fp_blocked)
            with open(host_fp_blocked, "a") as blocked_file:
                for url in self.blocked:
                    # TODO: refine, add www only if not in url
                    blocked_file.write(f"{self.redirect} {url}\n")
                    blocked_file.write(f"{self.redirect} www.{url}\n")
            return True
        except PermissionError:
            self._raise_permission_error()
            return False

    def block(self, notify=False):
        "Blocks all specified websites by replacing `hosts` file with `hosts.blocked`"
        try:
            copy(host_fp_blocked, host_fp)
        except PermissionError:
            self._raise_permission_error()
            return False
        if notify:
            self.notify("Websites blocked, enjoy your work")
        return "Websites blocked"

    def unblock(self, notify=False):
        "Unblocks all websites by restoring the original `hosts` file"
        try:
            copy(host_fp_copy, host_fp)
        except PermissionError:
            self._raise_permission_error()
            return False
        if notify:
            self.notify("All websites unblocked, have fun")
        return "Websites unblocked"

    def notify(self, message, title=APP_NAME, duration=5):
        "Sends notification to CLI and - if available - to GUI"
        print(message)
        if win_notify:
            win_notify.show_toast(title, message, duration=duration) # icon_path=ICON_PATH,

    def _raise_permission_error(self):
        self.notify("Permission Error. Please run the command line tool as ADMINISTRATOR.")

# Cell
if __name__=='__main__':
    blocker = Blocker()
    if blocker.adminrights:
        try: mode = sys.argv[1].lower()
        except: mode = 'none'

        if mode == 'block':
            blocker.block()
        elif mode == 'unblock':
            blocker.unblock()
        else:
            print("Run blocker.py with arguments BLOCK or UNBLOCK")


# AUTOGENERATED! DO NOT EDIT! File to edit: 00_beproductive.ipynb (unless otherwise specified).

__all__ = ['main', 'parse_arguments', 'in_notebook']

# Cell
from .blocker import *
from .pomodoro import *
import argparse
from time import sleep

# Cell
def main(action=None, time=None, break_time=None, pomodoros=None):
    if action == 'block' or action == None:
        blocker = Blocker()
        if blocker.adminrights:
            if time:
                blocker.block()
                blocker.notify(f'Websites blocked for {time} minutes.')
                sleep(time*60)
                blocker.unblock(notify=True)
            else:
                blocker.block(notify=True)
    elif action == 'unblock':
        blocker = Blocker()
        if blocker.adminrights:
            blocker.unblock(notify=True)
    elif action == 'pomodoro':
        pomodoro(work_time=time or WORK_TIME, break_time=break_time or BREAK_TIME, pomodoros=pomodoros or POMODOROS)

# Cell
def parse_arguments():
    """
    Parse arguments from shell. All arguments are optional.
    action: ['block', 'unblock', 'pomodoro']
    time: minutes
    break_time: minutes
    pomodoros: int

    Returns: Namespace
    """
    parser = argparse.ArgumentParser(description="Block addictive websites. Study with the Pomodoro technique.")
    parser.add_argument('action', nargs='?', choices=['block', 'unblock', 'pomodoro'], \
                        help='Block or unblock websites, or start a Pomodoro session. ')
    parser.add_argument('time', type=int, nargs='?', \
                        help='How many minutes should websites be blocked?')
    parser.add_argument('break_time', type=int, nargs='?', \
                        help='Length of the break between Pomodoros')
    parser.add_argument('pomodoros', type=int, nargs='?', \
                        help='Number of Pomodoros')
    args = parser.parse_args()
    return args

# Cell
def in_notebook():
    "Returns True if run in a notebook environment."
    try:
        from IPython import get_ipython
        if 'IPKernelApp' not in get_ipython().config:  # pragma: no cover
            return False
    except ImportError:
        return False
    return True

# Cell
if __name__ == '__main__' and not in_notebook():
    args = parse_arguments()
    main(args.action, args.time, args.break_time, args.pomodoros)


# AUTOGENERATED! DO NOT EDIT! File to edit: 02_pomodoro.ipynb (unless otherwise specified).

__all__ = ['WORK_TIME', 'BREAK_TIME', 'POMODOROS', 'pomodoro']

# Cell
from time import sleep
from .blocker import *
import sys

WORK_TIME = 25 # minutes
BREAK_TIME = 5 # minutes
POMODOROS = 4

# Cell
def pomodoro(work_time=WORK_TIME, break_time=BREAK_TIME, pomodoros=POMODOROS):
    blocker = Blocker()
    if not blocker.adminrights:
        return False
    turn = 1
    while turn <= pomodoros:
        if blocker.block():
            blocker.notify(f"Pomodoro no. {turn} of {pomodoros} started, work for {work_time} minutes")
        else:
            blocker.notify("An error occured. Please exit with ctrl+c")
        sleep(work_time*60)
        blocker.unblock()
        if turn < pomodoros:
            blocker.notify(f"Pomodoro no. {turn} ended, take a {break_time} minutes break")
            sleep(break_time*60)
        else:
            blocker.notify(f"Pomodoro session ended, take a longer break. All websites unblocked.", duration=10)
        turn += 1


# AUTOGENERATED! DO NOT EDIT! File to edit: 00_beproductive.ipynb (unless otherwise specified).

__all__ = ['main', 'parse_arguments']

# Cell
from .blocker import *
from .pomodoro import *
import argparse

# Cell
def main(action):
    print(f"Main! {action}")

# Cell
def parse_arguments():
    parser = argparse.ArgumentParser(description="Block addictive websites. Study with the Pomodoro technique.")
    parser.add_argument('action', help='block or unblock websites, or start a Pomodoro session', \
                        nargs='?', choices=['block', 'unblock', 'pomodoro'])
    args = parser.parse_args()
    print(args)
    return args

# Cell
if __name__ == '__main__':
    args = parse_arguments()
    main(args.action)


# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"main": "00_beproductive.ipynb",
         "parse_arguments": "00_beproductive.ipynb",
         "in_notebook": "00_beproductive.ipynb",
         "APP_NAME": "01_blocker.ipynb",
         "REDIRECT": "01_blocker.ipynb",
         "WIN_PATH": "01_blocker.ipynb",
         "LINUX_PATH": "01_blocker.ipynb",
         "ICON_PATH": "01_blocker.ipynb",
         "BLOCKLIST": "01_blocker.ipynb",
         "host_fp": "01_blocker.ipynb",
         "host_fp_copy": "01_blocker.ipynb",
         "host_fp_blocked": "01_blocker.ipynb",
         "Blocker": "01_blocker.ipynb",
         "WORK_TIME": "02_pomodoro.ipynb",
         "BREAK_TIME": "02_pomodoro.ipynb",
         "POMODOROS": "02_pomodoro.ipynb",
         "pomodoro": "02_pomodoro.ipynb"}

modules = ["__main__.py",
           "blocker.py",
           "pomodoro.py"]

doc_url = "https://johannesstutz.github.io/beproductive/"

git_url = "https://github.com/johannesstutz/beproductive/tree/master/"

def custom_doc_links(name): return None
